/* generic Channel Adjust
** ** (c) 2003 - Ernst Pech
** 
*/

#include "avisynth.h"
#include <math.h>
#include <time.h>
#include <stdio.h>


float EvalAVS(char * expression, float DefaultValue,  IScriptEnvironment* env) {
// returns a float value if the result is well defined
	AVSValue args[1];
	args[0] = expression;
	AVSValue result = env->Invoke("eval",AVSValue(args,1));
	
	if (result.Defined()) {
		if (result.IsFloat()) {
			return result.AsFloat();
		} else {
			env->ThrowError("Not a value");
		}
	} else {
		return DefaultValue;
	}
}


class AdjustY : public GenericVideoFilter {

	int bpp;
	int i;
	float res;
	char Y[256];
	bool eval_frame;
	unsigned char map[256];
	
public:
    AdjustY(PClip _child, const char _Y[], bool _eval_frame, IScriptEnvironment* env):GenericVideoFilter(_child){
	
		bpp = vi.BitsPerPixel()/8;

		if (vi.IsPlanar()) {
			env->ThrowError("Planar formats not supported!");
		};
		strcpy(Y, _Y);
		eval_frame = _eval_frame;

		env->SetVar("N",0.0);
		for (i=0; i<256; i++) {
			env->SetVar("Y",float(i));
			res = EvalAVS(Y,i,env);
			if (res<0) res=0;
			res = res + 0.5;	//rounding
			if (!(res<256)) res=255;
			map[i] = int(res);
		}

	}

		
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
};


PVideoFrame __stdcall AdjustY::GetFrame(int n, IScriptEnvironment* env) {

	PVideoFrame src = child->GetFrame(n, env);
	env->MakeWritable(&src);
	unsigned char * srcp = src->GetWritePtr();
	
	int srcheight = src->GetHeight();
	int srcwidth = src->GetRowSize() / bpp;
	int srcpitch = src->GetPitch();
	int srcoffset;

	int x;
	int y;

	if (eval_frame) {
		env->SetVar("N",float(n));
		for (i=0; i<256; i++) {
			env->SetVar("Y",float(i));
			res = EvalAVS(Y,i,env);
			if (res<0) res=0;
			res = res + 0.5;	//rounding
			if (!(res<256)) res=255;
			map[i] = int(res);
		}
	}

	for (y = 0 ; y < srcheight; y++) {	//loop over dst
		if (vi.IsRGB24()) {
			for (x = 0 ; x < srcwidth; x++) {
				srcoffset = srcpitch * y + x*bpp;
				srcp[srcoffset + 0] = map[srcp[srcoffset + 0]];
				srcp[srcoffset + 1] = map[srcp[srcoffset + 1]];
				srcp[srcoffset + 2] = map[srcp[srcoffset + 2]];
			}
		}

		if (vi.IsRGB32()) {	//alpha channel untouched
			for (x = 0 ; x < srcwidth; x++) {
				srcoffset = srcpitch * y + x*bpp;
				srcp[srcoffset + 0] = map[srcp[srcoffset + 0]];
				srcp[srcoffset + 1] = map[srcp[srcoffset + 1]];
				srcp[srcoffset + 2] = map[srcp[srcoffset + 2]];
			}
		}

		if (vi.IsYUY2()) {	//colour channels untouched
			for (x = 0 ; x < srcwidth; x++) {
				srcoffset = srcpitch * y + x*bpp;
				srcp[srcoffset + 0] = map[srcp[srcoffset + 0]];
			}
		}
	}
	return src;
}


AVSValue __cdecl Create_AdjustY(AVSValue args, void* user_data, IScriptEnvironment* env) {
	return new AdjustY(args[0].AsClip(), args[1].AsString(""), args[2].AsBool(false), env);
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
  env->AddFunction("AdjustY","c[y]s[eval_frame]b",Create_AdjustY,0);
  return 0;
}
